#########################################################################
#                                                                       #
#  This file is part of gazix.                                          #
#                                                                       #
#  gazix is free software: you can redistribute it and/or modify        #
#  it under the terms of the GNU General Public License as published by #
#  the Free Software Foundation, either version 3 of the License, or    #
#  (at your option) any later version.                                  #
#                                                                       #
#  gazix is distributed in the hope that it will be useful,             #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of       #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        # 
#  GNU General Public License for more details.                         #
#                                                                       #
#  You should have received a copy of the GNU General Public License    #
#  along with gazix. If not, see <https://www.gnu.org/licenses/>.       #
#                                                                       #
#########################################################################

gitlab_api_url = "--gitlab"
gitlab_api_url_src = "--gitlab_src"
gitlab_api_url_target = "--gitlab_target"

token = "--token"
token_src = "--token_src"
token_target = "--token_target"
token_trigger = "--token_trigger"

project = "--project"
project_src = "--project_src"
project_target = "--project_target"

MR = "--MR"

pipeline = "--pipeline"

variable_long = "--var"
variable_short = "-v"

reference_long = "--ref"
reference_short = "-r"

wait_long = "--wait"
wait_short = "-w"

help_short = "-h"
help_long = "--help"

unstable_short = "-u"
unstable_long = "--unstable"

gazix = "gazix"
